from paraview.web import venv  # Available in PV 5.10-RC2+
from paraview.simple import *

from pathlib import Path
from trame.app import get_server
from trame.widgets import vuetify, paraview, client
from trame.ui.vuetify import SinglePageLayout

# -----------------------------------------------------------------------------
# Trame setup
# -----------------------------------------------------------------------------

server = get_server()
state, ctrl = server.state, server.controller

# Preload paraview modules onto server
paraview.initialize(server)

# -----------------------------------------------------------------------------
# ParaView code
# -----------------------------------------------------------------------------


def load_data(**kwargs):
    # CLI
    args, _ = server.cli.parse_known_args()

    full_path = str(Path(args.data).resolve().absolute())
    # working_directory = str(Path(args.data).parent.resolve().absolute())

    # ParaView
    simple_volume_modelvtk = LegacyVTKReader(FileNames=[full_path])
    # simple.LoadState(
    #     full_path,
    #     data_directory=working_directory,
    #     restrict_to_data_directory=True,
    # )
    renderView1 = GetRenderView()
    simple_volume_modelvtkDisplay = Show(simple_volume_modelvtk, renderView1, 'UnstructuredGridRepresentation')

    simple_volume_modelvtkDisplay.Representation = 'Surface'
    simple_volume_modelvtkDisplay.ColorArrayName = [None, '']
    simple_volume_modelvtkDisplay.SelectTCoordArray = 'None'
    simple_volume_modelvtkDisplay.SelectNormalArray = 'None'
    simple_volume_modelvtkDisplay.SelectTangentArray = 'None'
    simple_volume_modelvtkDisplay.OSPRayScaleArray = 'Depth'
    simple_volume_modelvtkDisplay.OSPRayScaleFunction = 'PiecewiseFunction'
    simple_volume_modelvtkDisplay.SelectOrientationVectors = 'None'
    simple_volume_modelvtkDisplay.ScaleFactor = 58.2
    simple_volume_modelvtkDisplay.SelectScaleArray = 'Depth'
    simple_volume_modelvtkDisplay.GlyphType = 'Arrow'
    simple_volume_modelvtkDisplay.GlyphTableIndexArray = 'Depth'
    simple_volume_modelvtkDisplay.GaussianRadius = 2.91
    simple_volume_modelvtkDisplay.SetScaleArray = ['POINTS', 'Depth']
    simple_volume_modelvtkDisplay.ScaleTransferFunction = 'PiecewiseFunction'
    simple_volume_modelvtkDisplay.OpacityArray = ['POINTS', 'Depth']
    simple_volume_modelvtkDisplay.OpacityTransferFunction = 'PiecewiseFunction'
    simple_volume_modelvtkDisplay.DataAxesGrid = 'GridAxesRepresentation'
    simple_volume_modelvtkDisplay.PolarAxes = 'PolarAxesRepresentation'
    simple_volume_modelvtkDisplay.ScalarOpacityUnitDistance = 70.49820966433877
    simple_volume_modelvtkDisplay.OpacityArrayName = ['POINTS', 'Depth']
    simple_volume_modelvtkDisplay.SelectInputVectors = [None, '']
    simple_volume_modelvtkDisplay.WriteLog = ''
    simple_volume_modelvtkDisplay.ScaleTransferFunction.Points = [0.0, 0.0, 0.5, 0.0, 35.14495086669922, 1.0, 0.5, 0.0]
    simple_volume_modelvtkDisplay.OpacityTransferFunction.Points = [0.0, 0.0, 0.5, 0.0, 35.14495086669922, 1.0, 0.5, 0.0]
    renderView1.ResetCamera(False)
    ColorBy(simple_volume_modelvtkDisplay, ('CELLS', 'Material'))
    simple_volume_modelvtkDisplay.RescaleTransferFunctionToDataRange(True, False)
    simple_volume_modelvtkDisplay.SetScalarBarVisibility(renderView1, True)
    
    renderView1 = Render()
    ResetCamera()

    # HTML
    with SinglePageLayout(server) as layout:
        layout.icon.click = ctrl.view_reset_camera
        layout.title.set_text("ParaView Trame")

        with layout.content:
            with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
                html_view = paraview.VtkRemoteView(renderView1)
                ctrl.view_reset_camera = html_view.reset_camera
                ctrl.view_update = html_view.update


ctrl.on_server_ready.add(load_data)

# -----------------------------------------------------------------------------
# GUI
# -----------------------------------------------------------------------------

state.trame__title = "State Viewer"

with SinglePageLayout(server) as layout:
    layout.icon.click = ctrl.view_reset_camera
    layout.title.set_text("ParaView State Viewer")

    with layout.content:
        with vuetify.VContainer(fluid=True, classes="pa-0 fill-height"):
            client.Loading("Loading state")


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

if __name__ == "__main__":
    server.cli.add_argument("--data", help="Path to state file", dest="data")
    server.start()
